@@ -23,8 +23,8 @@
     series.stats$median <- colMedians(rwl2, na.rm=TRUE)
     series.stats$stdev <- colSds(rwl2, na.rm=TRUE)
     series.stats$skew <- apply(rwl2, 2, skew)
-    series.stats$sens1 <- apply(rwl2, 2, sens1)
-    series.stats$sens2 <- apply(rwl2, 2, sens2)
+    #series.stats$sens1 <- apply(rwl2, 2, sens1)
+    #series.stats$sens2 <- apply(rwl2, 2, sens2)
     series.stats$gini <- apply(rwl2, 2, gini.coef)
     series.stats$ar1 <- apply(rwl2, 2, acf1)
     seq.temp <- -seq_len(4)
